""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import logging
from operator import itemgetter

from django.db.models import Q
from rest_framework import viewsets, status
from rest_framework.response import Response

from expression_builder.models import Expression
from expression_builder.serializers import ExpressionMetaSerializer, ExpressionSerializer, ExpressionIdSerializer
from expression_builder.utility import insert_jinja_record

logger = logging.getLogger(__name__)


class ExpressionsView(viewsets.ModelViewSet):
    serializer_class = ExpressionSerializer

    def get_queryset(self):
        connector_name = self.request.query_params.get('connector_name')
        version = self.request.query_params.get('version')
        ref_only = self.request.query_params.get('ref_only', 'false')
        queryset = Expression.objects.all()
        if connector_name:
            queryset = queryset.filter(connector_name=connector_name)
        if version:
            queryset = queryset.filter(version=version)
        if ref_only == 'true':
            queryset = queryset.filter(Q(action_name__exact='') | Q(action_name=None))
        queryset = queryset.order_by('name', '-version').distinct('name')
        return queryset

    def list(self, request, *args, **kwargs):
        meta = request.query_params.get('meta', 'false')

        self.queryset = self.filter_queryset(self.get_queryset())
        if meta == 'true':
            self.serializer_class = ExpressionMetaSerializer
        serializer = self.get_serializer(self.queryset, many=True)

        return Response(form_functions_response(serializer.data), status=status.HTTP_200_OK)

    def create(self, request, *args, **kwargs):
        try:
            exp_cat_instances, exp_instances, created = insert_jinja_record(request.data)
        except Exception as e:
            return Response({'message': str(e)}, status=status.HTTP_400_BAD_REQUEST)

        return Response(form_functions_response(self.get_serializer(exp_instances, many=True).data),
                        status=status.HTTP_201_CREATED if created else status.HTTP_200_OK)


class ExpressionsBulkView(viewsets.ModelViewSet):
    queryset = Expression.objects.all()
    serializer_class = ExpressionSerializer

    def bulk_create_or_update(self, request, *args, **kwargs):
        try:
            input_data = request.data
            expressions_data = input_data.get('functions', [])
            if not expressions_data:
                error_message = "Functions are not provided in the request."
                logger.exception(error_message)
                return Response({"message": error_message}, status=status.HTTP_400_BAD_REQUEST)
            exp_instances = []
            result_status = [status.HTTP_200_OK]
            for exp_data in expressions_data:
                exp_cat_instances, exp_instance, created = \
                    insert_jinja_record(exp_data, result_status=result_status, is_bulk=True)
                if exp_instance:
                    for exp in exp_instance:
                        exp_instances.append(exp)

            self.serializer_class = ExpressionIdSerializer
            msg = "Functions have been created successfully!" if result_status[0] == status.HTTP_200_OK \
                else "Creation of one or more functions have been failed!"
            return Response({"message": msg,
                             "functions": self.get_serializer(exp_instances, many=True).data},
                            status=result_status[0])
        except Exception as e:
            error_message = "Error while creating the functions ERROR :: {0}".format(str(e))
            logger.exception(error_message)
            return Response({"message": error_message}, status=status.HTTP_400_BAD_REQUEST)

    def bulk_delete(self, request):
        try:
            connector_name = self.request.query_params.get('connector_name')
            version = self.request.query_params.get('version', '')
            payload = request.data
            exp_ids = payload.get('ids', [])
            if connector_name is None:
                exp_qs = Expression.objects.filter(id__in=exp_ids)
            else:
                exp_qs = Expression.objects.filter(connector_name=connector_name, version=version)
            exp_qs.delete()

            return Response({"message": "Functions have been deleted successfully!"},
                            status=status.HTTP_200_OK)
        except Exception as e:
            error_message = "Error while deleting the expression ERROR :: {0}".format(str(e))
            logger.exception(error_message)
            return Response({"message": error_message}, status=status.HTTP_400_BAD_REQUEST)


def form_functions_response(data):
    result = {}
    sorted_data = sorted(data, key=itemgetter('name'))
    for data in sorted_data:
        res = result.setdefault(data['name'], data)
        if res != data:
            res.get('operations', []).append(data['operations'][0])
    return result
