""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import os
import time
import shutil
import logging
import configparser

from os.path import join

from django.core.management.base import BaseCommand

ERROR_WHILE_MERGING = 'Error: Failed to merge the CONFIG.INI with new parameter :: %s, reverted back with original'

BASE_DIR = "/opt/cyops-workflow/sealab"
CONFIG_FILE = 'config.ini'
PARAMETER_CHANGE_FILE = 'parameters_changes.ini'
FORCE_CHANGE_FILE = 'force_config.ini'
BACKUP_FOLDER = "/tmp/cyops-workflow/sealab/config"


def backup_file(folder: str, file_name: str) -> str:
    file_path = join(folder, file_name)
    backup_path = file_path + '.bak'

    if os.path.exists(backup_path):
        milli_sec = int(round(time.time() * 1000))
        tmp_path = file_path = join(BACKUP_FOLDER, str(milli_sec) + "_" + file_name)
        os.makedirs(os.path.dirname(tmp_path), exist_ok=True)
        shutil.move(backup_path, tmp_path)

    shutil.copy(file_path, backup_path)
    logging.info(f"Backup created at '{backup_path}'.")
    return backup_path


class Command(BaseCommand):
    help = 'Add new configuration parameters to the CONFIG.INI while system upgrade'

    def handle(self, *args, **options):
        config_dir = join(BASE_DIR, 'sealab')
        ini_file_path = join(config_dir, CONFIG_FILE)
        parameter_change_file_path = join(config_dir, PARAMETER_CHANGE_FILE)
        force_change_param_file_path = join(config_dir, FORCE_CHANGE_FILE)

        if not os.path.isfile(ini_file_path):
            return self.stdout.write(self.style.ERROR('Error : CONFIG.INI file not present'))

        backup_ini_path = backup_file(config_dir, CONFIG_FILE)

        try:
            config = configparser.ConfigParser()
            config.read(ini_file_path)
            config, is_modified = self.merge_config(config, parameter_change_file_path)
            config, is_modified = self.merge_config(config, force_change_param_file_path, replace=True)

            if not is_modified:
                return self.stdout.write(self.style.SUCCESS('INFO: no changes in Config.ini'))

            with open(ini_file_path, 'w') as configfile:
                config.write(configfile)
        except Exception as e:
            shutil.copy(backup_ini_path, ini_file_path)
            return self.stdout.write(self.style.ERROR(ERROR_WHILE_MERGING, str(e)))

        return self.stdout.write(self.style.SUCCESS('Successfully added new parameters to CONFIG.INI.'))

    def merge_config(self, original_config, property_file_path, replace=False):
        config = configparser.ConfigParser()
        updated = False

        if not os.path.exists(property_file_path):
            return original_config, updated

        try:
            config.read(property_file_path)
            for section in config.sections():
                if section not in original_config:
                    original_config.add_section(section)
                    updated = True

                for property_name, new_value in config.items(section):
                    if (property_name not in original_config[section] or
                            (property_name in original_config[section] and replace)):
                        original_config[section][property_name] = new_value
                        updated = True
        except Exception as e:
            print(self.style.ERROR(f'Error: while merging parameters with config.ini : {str(e)}'))
            self.stdout.write(self.style.ERROR(f'Error: while merging parameters with config.ini : {str(e)}'))

        return original_config, updated
